object schedule {
  infix fun meeting(block: Meeting.() -> Unit) = print(Meeting().apply(block))
}

class Meeting {
  var meetingName = ""
  var startTime = IntRange.EMPTY
  var endTime = IntRange.EMPTY
  var scheduledOn = ""
  var attending = ""
  //within the Meeting class
  val assign = this
  //within the Meeting class
  val starts = Starts()
  val ends = Ends()
  val on = On()
  val participants = Participants()  

  infix fun name(name: String) {
    meetingName = name
  }

  inner class Starts {
    infix fun at(time: IntRange) {
      startTime = time
    }
  }

  inner class Ends {
    infix fun at(time: IntRange) {
      endTime = time
    }
  }
  
  inner class On {
    infix fun date(day: Int) = DateCreator(day, this@Meeting)
  }
  
  class DateCreator(val day: Int, val meeting: Meeting) {
    private fun setScheduledOn(month: Int, year: Int) {
      meeting.scheduledOn =
        java.time.LocalDate.of(year, month, day).toString()
    }
    
    infix fun January(year: Int) = setScheduledOn(1, year)
    infix fun February(year: Int) = setScheduledOn(2, year)
    infix fun March(year: Int) = setScheduledOn(3, year)
    //... April, May,... December
  }

  inner class Participants {
    infix fun include(name: String): Participants = apply {
      attending = name
    }
    
    infix fun and(name: String): Participants = apply {
      attending = "$attending, $name"
    }
  }  

  override fun toString() = """Meeting: $meetingName
  |Starts at ${startTime.start}:${startTime.endInclusive}
  |Ends at ${endTime.start}:${endTime.endInclusive}
  |On $scheduledOn
  |Participants: ${attending}
  |
  """.trimMargin()
}
